/*

  Stroke.java
  KanjiDraw

 Created by Julio Gorge on 07/11/2006.
 Copyright 2006-2007 LemonTeam.
 
 Kana DS is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Kana DS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Kana DS; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Line2D;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.geom.Point2D;
import java.util.ArrayList;

public class Stroke extends ArrayList
{
	// This variable has an impact in the complexity of the strokes that can be represented.
	// It might also slightly affect performance of gesture recognition later on.
	
	public static final int PointsPerStroke = 21;
	private static final float StrokeWidth = 15.f;
	
	// Clone
	
	public Object clone()
	{
		return super.clone();
	}
	
	
	// Return length of stroke
	
	float length()
	{ 
		int size = size();
		
		if ( size <= 1 )
			return 0.f;
		
		float len = 0.f;

		Point2D startPoint = (Point2D) get( 0 );
		Point2D endPoint;
		
		for ( int i = 1; i < size; i ++ )
		{
			endPoint = (Point2D) get( i );
			
			float dx = (float)( endPoint.getX() - startPoint.getX() );
			float dy = (float)( endPoint.getY() - startPoint.getY() );
			
			len += Math.sqrt( dx * dx + dy * dy );
			
			startPoint = endPoint;
		}
			
		return len;
	}


	// Normalize the spacing between the 2D points that define the stroke

	public Stroke normalizeSpacing()
	{
		Stroke newStroke = new Stroke();
		
		float newSegmentLen = length();
		
		if ( size() <= 1.f || newSegmentLen <= 0.f )
			return null;
			
		newSegmentLen /= ( PointsPerStroke - 1 );
		
		newStroke.add( get( 0 ) );
		
		Point2D startPt = (Point2D)get( 0 );
		Point2D endPt = (Point2D)get( 0 );
		
		//Distance along old stroke at the end of the current segment
		float endOldDist     = 0.0f;
		
		//Distance along the old stroke at the beginning of the current segment
		float startOldDist   = 0.0f;
		
		//Distance along new stroke
		float newDist        = 0.0f;
		
		//Length of the current segment (on the old stroke)
		float currSegmentLen = 0.0f;
				
		int i = 1;
		
		while ( true )
		{
			float excess = endOldDist - newDist;
			
			if ( excess >= newSegmentLen )
			{
				 newDist += newSegmentLen;
				 float ratio = ( newDist - startOldDist ) / currSegmentLen;
				 
				 Point2D newPt = new Point2D.Float();
				 
				 newPt.setLocation( (float)( endPt.getX() - startPt.getX() ) * ratio + (float)startPt.getX(),
									(float)( endPt.getY() - startPt.getY() ) * ratio + (float)startPt.getY() );
				 
				 newStroke.add( (Point2D)newPt );
			}
			else
			{
				if ( i == size() )
					break;
					
				 //Store the start of the current segment
				 startPt = endPt;
				 endPt = (Point2D)get( i ); //Get next point
				 ++i;
				 float dx = (float)( endPt.getX() - startPt.getX() );
				 float dy = (float)( endPt.getY() - startPt.getY() );

				 //Start accumulated distance (along the old stroke)
				 //at the beginning of the segment
				 
				 startOldDist = endOldDist;
				 
				 //Add the length of the current segment to the
				 //total accumulated length
				 currSegmentLen = (float)Math.sqrt( dx * dx + dy * dy);
				 endOldDist += currSegmentLen;
			 
			}

		}
		
		if ( newStroke.size() < PointsPerStroke )
		{
			newStroke.add( new Point2D.Float( (float)endPt.getX(), (float)endPt.getY() ) );
		}
		
	   
	   return newStroke;
	}
	
	
	// Draw stroke using a Graphics2D interface
	public void draw( Graphics2D g )
	{
		draw( g, 0, 0, 1.f, 1.f );
	}
	
	public void draw( Graphics2D g, int offsetX, int offsetY, float scaleX, float scaleY )
	{
		int ptCount = size();
		
		if ( ptCount > 1 )
		{
			g.setColor( Color.black );
			
			g.setStroke(new BasicStroke( StrokeWidth * scaleX, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER ) );

		
			Point2D.Float ptStart = (Point2D.Float)get( 1 );
			Point2D.Float ptEnd;
			
			for ( int j = 2; j < ptCount; j ++ )
			{
				ptEnd = (Point2D.Float)get( j );
				
				g.drawLine( (int)( offsetX + ptStart.getX() * scaleX ),
							(int)( offsetY + ptStart.getY() * scaleY ),
							(int)( offsetX + ptEnd.getX() * scaleX ),
							(int)( offsetY + ptEnd.getY() * scaleY ) );
				
				ptStart = ptEnd;
			}
		}
	}
	
}
